"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isOptional = exports.hasInitializer = exports.serializeTypeParameters = exports.serializeMethods = exports.serializeConstructors = exports.isModifier = exports.getMemberModifier = exports.getOriginalFileOriginalType = exports.getHeritageClauseNames = exports.create = exports.isNodeExported = void 0;
var typescript_1 = __importDefault(require("typescript"));
var ClassFactory = __importStar(require("./ClassFactory"));
var EnumFactory = __importStar(require("./EnumFactory"));
var InterfaceFactory = __importStar(require("./InterfaceFactory"));
var MethodFactory = __importStar(require("./MethodFactory"));
var NamespaceFactory = __importStar(require("./NamespaceFactory"));
var PropertyFactory = __importStar(require("./PropertyFactory"));
var TypeParameterFactory = __importStar(require("./TypeParameterFactory"));
function isNodeExported(node) {
    return (node.flags & typescript_1.default.ModifierFlags.Export) !== 0 ||
        node.parent.kind === typescript_1.default.SyntaxKind.SourceFile ||
        node.parent.kind === typescript_1.default.SyntaxKind.ModuleBlock;
}
exports.isNodeExported = isNodeExported;
function create(fileName, node, checker) {
    var componentComposites = [];
    typescript_1.default.forEachChild(node, function (childNode) {
        if (!isNodeExported(childNode)) {
            return;
        }
        if (childNode.kind === typescript_1.default.SyntaxKind.ClassDeclaration) {
            var currentNode = childNode;
            if (currentNode.name === undefined) {
                return;
            }
            var classSymbol = checker.getSymbolAtLocation(currentNode.name);
            if (classSymbol === undefined) {
                return;
            }
            componentComposites.push(ClassFactory.create(fileName, classSymbol, checker));
        }
        else if (childNode.kind === typescript_1.default.SyntaxKind.InterfaceDeclaration) {
            var currentNode = childNode;
            var interfaceSymbol = checker.getSymbolAtLocation(currentNode.name);
            if (interfaceSymbol === undefined) {
                return;
            }
            componentComposites.push(InterfaceFactory.create(interfaceSymbol, checker));
        }
        else if (childNode.kind === typescript_1.default.SyntaxKind.ModuleDeclaration) {
            var currentNode = childNode;
            var namespaceSymbol = checker.getSymbolAtLocation(currentNode.name);
            if (namespaceSymbol === undefined) {
                return;
            }
            componentComposites.push(NamespaceFactory.create(fileName, namespaceSymbol, checker));
        }
        else if (childNode.kind === typescript_1.default.SyntaxKind.EnumDeclaration) {
            var currentNode = childNode;
            var enumSymbol = checker.getSymbolAtLocation(currentNode.name);
            if (enumSymbol === undefined) {
                return;
            }
            componentComposites.push(EnumFactory.create(enumSymbol));
            return;
        }
        else if (childNode.kind === typescript_1.default.SyntaxKind.FunctionDeclaration) {
            var currentNode = childNode;
            if (currentNode.name === undefined) {
                return;
            }
            var functionSymbol = checker.getSymbolAtLocation(currentNode.name);
            if (functionSymbol === undefined) {
                return;
            }
            componentComposites.push(MethodFactory.create(functionSymbol, currentNode, checker));
        }
    });
    return componentComposites;
}
exports.create = create;
function getModifier(modifiers) {
    for (var _i = 0, modifiers_1 = modifiers; _i < modifiers_1.length; _i++) {
        var modifier = modifiers_1[_i];
        if (modifier.kind === typescript_1.default.SyntaxKind.PrivateKeyword) {
            return 'private';
        }
        if (modifier.kind === typescript_1.default.SyntaxKind.PublicKeyword) {
            return 'public';
        }
        if (modifier.kind === typescript_1.default.SyntaxKind.ProtectedKeyword) {
            return 'protected';
        }
    }
    return 'public';
}
function getHeritageClauseNames(heritageClause, checker) {
    return heritageClause.types.map(function (nodeObject) {
        var symbolAtLocation = checker.getSymbolAtLocation(nodeObject.expression);
        if (symbolAtLocation !== undefined) {
            var ogFile = getOriginalFile(symbolAtLocation, checker);
            return [checker.getFullyQualifiedName(symbolAtLocation), ogFile];
        }
        return ['', ''];
    });
}
exports.getHeritageClauseNames = getHeritageClauseNames;
function getOriginalFile(typeSymbol, checker) {
    var _a, _b;
    var deAliasSymbol;
    if ((typeSymbol.flags & typescript_1.default.SymbolFlags.Alias) !== 0) {
        deAliasSymbol = checker.getAliasedSymbol(typeSymbol);
    }
    else {
        deAliasSymbol = typeSymbol;
    }
    return (_b = (_a = deAliasSymbol.declarations) === null || _a === void 0 ? void 0 : _a[0].getSourceFile().fileName) !== null && _b !== void 0 ? _b : '';
}
function getOriginalFileOriginalType(tsType, checker) {
    if (tsType === undefined || checker === undefined) {
        return '';
    }
    var deParameterType = tsType;
    var typeSymbol = tsType.getSymbol();
    while ((typeSymbol === null || typeSymbol === void 0 ? void 0 : typeSymbol.name) === 'Array') {
        deParameterType = checker.getTypeArguments(deParameterType)[0];
        typeSymbol = deParameterType.getSymbol();
    }
    if (typeSymbol === undefined) {
        return '';
    }
    return getOriginalFile(typeSymbol, checker);
}
exports.getOriginalFileOriginalType = getOriginalFileOriginalType;
function isConstructor(declaration) {
    return declaration.kind === typescript_1.default.SyntaxKind.Constructor;
}
function isMethod(declaration) {
    return declaration.kind === typescript_1.default.SyntaxKind.MethodDeclaration ||
        declaration.kind === typescript_1.default.SyntaxKind.MethodSignature;
}
function isProperty(declaration) {
    return declaration.kind === typescript_1.default.SyntaxKind.PropertySignature ||
        declaration.kind === typescript_1.default.SyntaxKind.PropertyDeclaration ||
        declaration.kind === typescript_1.default.SyntaxKind.GetAccessor ||
        declaration.kind === typescript_1.default.SyntaxKind.SetAccessor ||
        declaration.kind === typescript_1.default.SyntaxKind.Parameter;
}
function isTypeParameter(declaration) {
    return declaration.kind === typescript_1.default.SyntaxKind.TypeParameter;
}
function getMemberModifier(memberDeclaration) {
    if (!typescript_1.default.canHaveModifiers(memberDeclaration)) {
        return 'public';
    }
    var memberModifiers = typescript_1.default.getModifiers(memberDeclaration);
    if (memberModifiers === undefined) {
        return 'public';
    }
    return getModifier(memberModifiers);
}
exports.getMemberModifier = getMemberModifier;
function isModifier(memberDeclaration, modifierKind) {
    if (!typescript_1.default.canHaveModifiers(memberDeclaration)) {
        return false;
    }
    var memberModifiers = typescript_1.default.getModifiers(memberDeclaration);
    if (memberModifiers !== undefined) {
        for (var _i = 0, memberModifiers_1 = memberModifiers; _i < memberModifiers_1.length; _i++) {
            var memberModifier = memberModifiers_1[_i];
            if (memberModifier.kind === modifierKind) {
                return true;
            }
        }
    }
    return false;
}
exports.isModifier = isModifier;
function serializeConstructors(memberSymbols, checker) {
    var result = [];
    if (memberSymbols !== undefined) {
        memberSymbols.forEach(function (memberSymbol) {
            var memberDeclarations = memberSymbol.getDeclarations();
            if (memberDeclarations === undefined) {
                return;
            }
            memberDeclarations.forEach(function (memberDeclaration) {
                if (isConstructor(memberDeclaration)) {
                    result.push(MethodFactory.create(memberSymbol, memberDeclaration, checker));
                }
            });
        });
    }
    return result;
}
exports.serializeConstructors = serializeConstructors;
function serializeMethods(memberSymbols, checker) {
    var result = [];
    if (memberSymbols !== undefined) {
        memberSymbols.forEach(function (memberSymbol) {
            var memberDeclarations = memberSymbol.getDeclarations();
            if (memberDeclarations === undefined) {
                return;
            }
            memberDeclarations.forEach(function (memberDeclaration) {
                if (isMethod(memberDeclaration)) {
                    result.push(MethodFactory.create(memberSymbol, memberDeclaration, checker));
                }
                else if (isProperty(memberDeclaration)) {
                    result.push(PropertyFactory.create(memberSymbol, memberDeclaration, checker));
                }
            });
        });
    }
    return result;
}
exports.serializeMethods = serializeMethods;
function serializeTypeParameters(memberSymbols, checker) {
    var result = [];
    if (memberSymbols !== undefined) {
        memberSymbols.forEach(function (memberSymbol) {
            var memberDeclarations = memberSymbol.getDeclarations();
            if (memberDeclarations === undefined) {
                return;
            }
            memberDeclarations.forEach(function (memberDeclaration) {
                if (isTypeParameter(memberDeclaration)) {
                    result.push(TypeParameterFactory.create(memberSymbol, memberDeclaration, checker));
                }
            });
        });
    }
    return result;
}
exports.serializeTypeParameters = serializeTypeParameters;
function hasInitializer(declaration) {
    return declaration.initializer !== undefined;
}
exports.hasInitializer = hasInitializer;
function isOptional(declaration) {
    return declaration.questionToken !== undefined;
}
exports.isOptional = isOptional;
